# Contributing

Users can contribute to this project by reporting issues or submitting changes.
Please follow the guidelines below to help assist maintainers for this project.

## Reporting issues

Bugs and enhancements for this project can be submitted to this project's issue
tracker. Before submitting an issue, consider the following:

- Check if the same or related issue exists. If so, feel free to add additional
  information to enhance the issue's content.
- Including versions helps (such as this project's version, Python's version,
  etc.).
- For more long and detailed logging information, it is better to attach the
  information opposed to dumping the entire contents into the issue's body.

## Submitting changes

Contributions can be provided in the form of Git-formatted patches by Email or
submitted through this project's pull request system. Before submitting changes,
consider the following:

- Always test any changes provided.
- Commit messages for patches requires a summary, a body (describing why the
  change is being made along with any other additional information) and
  metadata. It is a requirement that any submitted changes provided follow the
  Developer Certificate of Origin. This is indicated with the inclusion of the
  `Signed-off-by` tag in the metadata. Commit messages with only "bug fix" or
  "addressing comments in review" are not helpful when reflecting on the history
  of changes for this project.
- A patch should attempt to have a narrow scope as well as be complete.
- Be aware that while maintainers will help introduce changes into this project,
  the process may take some time. Please be patient.

## Quirks

The following outlines a series of quirks when contributing:

- Commit history is important. If a pull request is submitted, any commit should
  have meaningful message which describes the purpose of the change. There may
  be times where a maintainer or other contributers may request changes to a
  pull request. Individuals working on a pull request may either reset/re-form
  their commits or stack changes on the branch. If a developer wishes to stack
  changes onto a pull request, it is important that the newly added commit
  messages are describing the stacked change(s) as well. Adding a new commit
  with a message "updated from review" is not useful when observing history
  outside of the pull request.
- The recommended maximum line length is 80 characters for this project over
  PEP 8's recommendation of 79 characters. This preference is not enforced by
  the tox configuration as there may be valid reasons for some implementation to
  exceed 80 characters in length. That being said, please aim for any changes
  introduced to not exceed the 80 character limit when possible.
- PEP 8 "two blank lines" recommendations are not preferred in this project.
  Changes submitted with two blank lines may be accepted, but may also be
  "cleaned up" in future changes.

## Outro

Thanks for considering to be a participant to this project.
